package zendesk.messaging.android

import android.content.Context
import zendesk.android.Zendesk
import zendesk.android.internal.ZendeskError
import zendesk.messaging.android.internal.WrapperMessaging
import zendesk.android.messaging.Messaging as ZendeskMessaging

/**
 * Public entry point of the SDK. To use [Messaging] out of the box you must call [Messaging.initialize] first and once
 * [SuccessCallback] is called the SDK instance will be available through [Messaging.instance].
 *
 * ```kotlin
 * Messaging.initialize(
 *     context = context,
 *     channelKey = "{your-channel-key}",
 *     successCallback = {
 *         // The initialization was successful
 *     },
 *     failureCallback = { cause ->
 *         // The initialization failed and "cause" is an optional "MessagingError"
 *     }
 * )
 * ```
 *
 * After [Messaging.initialize] completes successfully, calling [Messaging.instance] will return an instance of
 * [Messaging] which you can use to perform SDK actions such as showing the conversation screen.
 *
 * ```kotlin
 * Messaging.instance().showMessaging(context)
 * ```
 *
 * @see Messaging.initialize
 * @see Messaging.instance
 */
@Deprecated(
    message = "Use Zendesk SDK to obtain an instance of Messaging",
    replaceWith = ReplaceWith(
        expression = "Messaging",
        imports = ["zendesk.android.messaging.Messaging"]
    )
)
public interface Messaging : ZendeskMessaging {

    public companion object {

        /**
         * Initializes [Messaging] for the given [channelKey], calling [successCallback] if the initialization
         * completes successfully and [failureCallback] if it fails.
         *
         * After a successful call to [initialize], calling it again will simply call [successCallback] immediately.
         *
         * This function is synchronized and won't run concurrently if called more than once at the same time.
         *
         * @param context an instance of [Context]. It will be retained for as long as this instance of [Messaging]
         * exists so avoid providing a short lived [Context] which could result in memory leaks
         * @param channelKey the key of your Messaging for Android channel
         * @param successCallback an implementation of [SuccessCallback] to be notified of a successful result
         * @param failureCallback an implementation of [FailureCallback] to be notified of a failure
         */
        @JvmStatic
        @Deprecated(
            message = "Please use Zendesk.initialize instead",
            replaceWith = ReplaceWith(
                expression = """Zendesk.initialize(context, channelKey, successCallback, failureCallback,
 DefaultMessagingFactory())""",
                imports = ["zendesk.android.Zendesk"]
            )
        )
        public fun initialize(
            context: Context,
            channelKey: String,
            successCallback: SuccessCallback<Messaging>,
            failureCallback: FailureCallback<MessagingError>,
        ) {
            Zendesk.initialize(
                context = context,
                channelKey = channelKey,
                messagingFactory = DefaultMessagingFactory(),
                successCallback = {
                    val wrapperMessaging = WrapperMessaging(it)
                    successCallback.onSuccess(wrapperMessaging)
                },
                failureCallback = {
                    val messagingError = when (it) {
                        ZendeskError.AccountNotFound -> MessagingError.AccountNotFound
                        is ZendeskError.FailedToInitialize -> MessagingError.FailedToInitialize(it.cause)
                        ZendeskError.InvalidChannelKey -> MessagingError.InvalidChannelKey
                        ZendeskError.MissingConfiguration -> MessagingError.MissingConfiguration
                        ZendeskError.NotInitialized -> MessagingError.NotInitialized
                        ZendeskError.SdkNotEnabled -> MessagingError.SdkNotEnabled
                        else -> null
                    }
                    failureCallback.onFailure(messagingError)
                },
            )
        }

        /**
         * Retrieves the current instance of [Messaging].
         *
         * If [Messaging.initialize] was not called or haven't completed yet, this method will return a stubbed version
         * of [Messaging] that will only output logs when any of its methods are called.
         *
         * @return the instance of [Messaging] if [Messaging.initialize] completed successfully, a stub otherwise
         *
         * @see Messaging.initialize
         */
        @JvmStatic
        @Deprecated(
            message = "Please use Zendesk#messaging instead",
            replaceWith = ReplaceWith(
                expression = "Zendesk.instance().messaging()",
                imports = ["zendesk.android.Zendesk"]
            )
        )
        public fun instance(): Messaging = WrapperMessaging(Zendesk.instance)

        /**
         * Invalidates the current instance of [Messaging].
         *
         * After calling this method you will have to call [Messaging.initialize] again if you would like to use
         * [Messaging].
         */
        @JvmStatic
        @Deprecated(
            message = "Please use Zendesk.invalidate instead",
            replaceWith = ReplaceWith(
                expression = "Zendesk.invalidate()",
                imports = ["zendesk.android.Zendesk"]
            )
        )
        public fun invalidate() {
            Zendesk.invalidate()
        }

        /**
         * Sets the [MessagingDelegate] that should be used by [Messaging].
         *
         * @param messagingDelegate an implementation of [MessagingDelegate], or null if you want to remove
         * one that was previously set
         */
        @JvmStatic
        @Deprecated(
            message = "Please use Messaging.setDelegate",
            replaceWith = ReplaceWith(
                expression = "Messaging.setDelegate(messagingDelegate)",
                imports = ["zendesk.android.messaging.Messaging"]
            )
        )
        public fun setDelegate(messagingDelegate: MessagingDelegate?) {
            ZendeskMessaging.setDelegate(messagingDelegate)
        }
    }
}
