package zendesk.messaging.android.push

import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.os.Build
import androidx.annotation.DrawableRes
import androidx.annotation.RequiresApi
import androidx.core.app.NotificationCompat
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.filter
import kotlinx.coroutines.launch
import zendesk.logger.Logger
import zendesk.messaging.R
import zendesk.messaging.android.internal.MessagingBuildConfig
import zendesk.messaging.android.internal.ScreenStateStore
import zendesk.messaging.android.internal.conversationscreen.ConversationActivity
import zendesk.messaging.android.push.PushResponsibility.MESSAGING_SHOULD_DISPLAY
import zendesk.messaging.android.push.PushResponsibility.MESSAGING_SHOULD_NOT_DISPLAY
import zendesk.messaging.android.push.PushResponsibility.NOT_FROM_MESSAGING
import zendesk.messaging.android.push.internal.NotificationBuilder
import zendesk.messaging.android.push.internal.NotificationProcessor
import zendesk.messaging.android.push.internal.NotificationProcessorFactory

/**
 * Public entry point to use push notifications with [zendesk.android.messaging.Messaging].
 */
public object PushNotifications {

    private const val LOG_TAG = "PushNotifications"
    private const val NOTIFICATION_KEY = "smoochNotification"
    private const val MESSAGING_NOTIFICATION_CHANNEL_ID = "MESSAGING_NOTIFICATION_CHANNEL_ID"

    private var notificationProcessor: NotificationProcessor? = null
    private val coroutineScope = CoroutineScope(context = Dispatchers.Default + SupervisorJob())

    /**
     * Backing mutable version of [pushNotificationToken].
     */
    private val mutablePushNotificationToken = MutableStateFlow("")

    /**
     * [Flow] that allows us to internally handle changes to a static
     * field from a [zendesk.android.messaging.Messaging] instance.
     */
    internal val pushNotificationToken: Flow<String>
        get() = mutablePushNotificationToken.filter { it.isNotEmpty() }

    @DrawableRes
    internal var smallNotificationIconId: Int = R.drawable.zma_default_notification_icon

    /**
     * Performs any necessary setup for push notifications to work correctly in the SDK.
     *
     * This includes:
     * - Creating a notification channel if the device is running on or above Android Oreo
     * - Creating an instance of [NotificationProcessor]
     *
     * @param context an instance of [Context]
     */
    private fun initialize(context: Context) {
        // If running on Oreo or above we create the notification channel
        if (MessagingBuildConfig.atLeastAndroid26()) {
            (context.getSystemService(Context.NOTIFICATION_SERVICE) as? NotificationManager)
                ?.createNotificationChannel(
                    buildChannel(channelName = context.getString(R.string.zma_notification_channel_name))
                )
        }

        notificationProcessor = NotificationProcessorFactory.create()
    }

    /**
     * Sets fields on this object back to their default state. This function is intended to be used for
     * testing only.
     */
    internal fun reset() {
        smallNotificationIconId = R.drawable.zma_default_notification_icon
        notificationProcessor = null
    }

    /**
     * Builds a notification channel for the SDK to receive push notifications on Android Oreo and above.
     * Calling this function multiple times will always return the same [NotificationChannel], but a
     * localised [channelName] can be provided and that will be updated.
     *
     * @param channelName the name of the notification channel
     */
    @RequiresApi(Build.VERSION_CODES.O)
    private fun buildChannel(channelName: String): NotificationChannel = NotificationChannel(
        MESSAGING_NOTIFICATION_CHANNEL_ID,
        channelName,
        NotificationManager.IMPORTANCE_HIGH,
    ).apply {
        enableVibration(true)
        enableLights(true)
    }

    /**
     * Updates the [pushNotificationToken] associated with this device to receive push notifications.
     *
     * This function can be called at any time. The [pushNotificationToken] is only be updated once
     * a [zendesk.conversationkit.android.model.User] becomes available. If one was not available when this
     * method was called, then the SDK will defer the update automatically.
     *
     * @param pushNotificationToken the device token for receiving push notifications
     */
    @JvmStatic
    public fun updatePushNotificationToken(pushNotificationToken: String) {
        mutablePushNotificationToken.value = pushNotificationToken
    }

    /**
     * When receiving a push notification payload, this method can be used to check if
     * it belongs to Messaging and if it can be displayed.
     *
     * @param messageData the [data][Map] received from the push notification
     *
     * @return an instance of [PushResponsibility]
     */
    @JvmStatic
    public fun shouldBeDisplayed(messageData: Map<String, String>): PushResponsibility = when {
        !messageData[NOTIFICATION_KEY].toBoolean() -> NOT_FROM_MESSAGING
        ScreenStateStore.currentlyVisibleScreen.value is ConversationActivity -> MESSAGING_SHOULD_NOT_DISPLAY
        else -> MESSAGING_SHOULD_DISPLAY
    }

    /**
     * Displays the given [messageData] as a push notification in the system tray if the push belongs
     * to Messaging and the SDK is able to display it.
     *
     * If the message doesn't belong to Messaging or the SDK is unable to display the message then
     * this method will return early, without displaying the push notification.
     *
     * @param context an instance of [Context]
     * @param messageData the [data][Map] received from the push notification
     */
    @JvmStatic
    public fun displayNotification(context: Context, messageData: Map<String, String>) {
        when (shouldBeDisplayed(messageData)) {
            NOT_FROM_MESSAGING -> {
                Logger.w(LOG_TAG, "Cannot display notification because it doesn't belong to Messaging")
                return
            }
            MESSAGING_SHOULD_NOT_DISPLAY,
            MESSAGING_SHOULD_DISPLAY,
            -> {
                initialize(context)
                coroutineScope.launch {
                    notificationProcessor?.display(
                        context = context,
                        messageData = messageData,
                        notificationBuilder = NotificationBuilder(
                            NotificationCompat.Builder(context, MESSAGING_NOTIFICATION_CHANNEL_ID),
                            context = context,
                        ),
                        smallIconId = smallNotificationIconId,
                    ) ?: Logger.w(LOG_TAG, "Cannot display notification because internal push setup has not completed")
                }
            }
        }
    }

    /**
     * Sets the [DrawableRes] ID to be used as a small icon in push notifications. Passing a null
     * value will reset the icon ID back to the default value.
     *
     * @param smallIconId the [DrawableRes] ID of the drawable to be used as the small icon
     */
    @JvmStatic
    public fun setNotificationSmallIconId(@DrawableRes smallIconId: Int?) {
        this.smallNotificationIconId = smallIconId ?: R.drawable.zma_default_notification_icon
    }
}
